package geniusweb.opponentmodel.bayesian;

import static org.junit.Assert.assertEquals;

import java.math.BigDecimal;
import java.util.Arrays;
import java.util.List;

import org.junit.Before;
import org.junit.Test;

import geniusweb.issuevalue.NumberValue;
import tudelft.utilities.junit.GeneralTests;

public class TriangleValueSetUtilTest
		extends GeneralTests<TriangleValueSetUtilities> {

	private static final double MAXE = 0.00000001;
	private static final BigDecimal N0 = BigDecimal.ZERO;
	private static final BigDecimal N01 = new BigDecimal("0.1");
	private static final BigDecimal N02 = new BigDecimal("0.2");
	private static final BigDecimal N03 = new BigDecimal("0.3");
	private static final BigDecimal N04 = new BigDecimal("0.4");
	private static final BigDecimal N05 = new BigDecimal("0.5");
	private static final BigDecimal N07 = new BigDecimal("0.7");
	private static final BigDecimal N09 = new BigDecimal("0.9");
	private static final BigDecimal N1 = BigDecimal.ONE;

	private TriangleValueSetUtilities vset1, vset1a, vset2, vset3;

	@Before
	public void before() {
		vset1 = new TriangleValueSetUtilities(N01, N02, N04, N05, N07, N09);
		vset1a = new TriangleValueSetUtilities(N01, N02, N04, N05, N07, N09);
		vset2 = new TriangleValueSetUtilities(N01, N02, N01, N05, N07, N09);
		vset3 = new TriangleValueSetUtilities(N01, N02, N07, N05, N07, N09);
	}

	@Override
	public List<List<TriangleValueSetUtilities>> getGeneralTestData() {
		return Arrays.asList(Arrays.asList(vset1, vset1a), Arrays.asList(vset2),
				Arrays.asList(vset3));
	}

	@Override
	public List<String> getGeneralTestStrings() {
		return Arrays.asList(
				"TriangleValueSetUtilities.0.1->0.2,0.4->0.5,0.7->0.9.",
				"TriangleValueSetUtilities.0.1->0.2,0.1->0.5,0.7->0.9.",
				"TriangleValueSetUtilities.0.1->0.2,0.7->0.5,0.7->0.9.");
	}

	@Test
	public void testInterpolation() {
		assertEquals(0, vset1.getUtility(new NumberValue(N0)).doubleValue(),
				MAXE);
		assertEquals(0.2, vset1.getUtility(new NumberValue(N01)).doubleValue(),
				MAXE);
		assertEquals(0.3 /* 2 + 1/2*3 */,
				vset1.getUtility(new NumberValue(N02)).doubleValue(), MAXE);
		assertEquals(0.5, vset1.getUtility(new NumberValue(N04)).doubleValue(),
				MAXE);
		assertEquals(0.9, vset1.getUtility(new NumberValue(N07)).doubleValue(),
				MAXE);
		assertEquals(0, vset1.getUtility(new NumberValue(N09)).doubleValue(),
				MAXE);

	}

	@Test
	public void testMidEqualsLow() {
		assertEquals(0, vset2.getUtility(new NumberValue(N0)).doubleValue(),
				MAXE);
		assertEquals(0.5, vset2.getUtility(new NumberValue(N01)).doubleValue(),
				MAXE);
		assertEquals(0.5 + 0.4 / 6,
				vset2.getUtility(new NumberValue(N02)).doubleValue(), MAXE);
		assertEquals(0.9, vset2.getUtility(new NumberValue(N07)).doubleValue(),
				MAXE);
		assertEquals(0, vset2.getUtility(new NumberValue(N09)).doubleValue(),
				MAXE);

	}

	@Test
	public void testMidEqualsHigh() {
		assertEquals(0, vset3.getUtility(new NumberValue(N0)).doubleValue(),
				MAXE);
		assertEquals(0.2, vset3.getUtility(new NumberValue(N01)).doubleValue(),
				MAXE);
		assertEquals(0.2 + 2 * 0.3 / 6,
				vset3.getUtility(new NumberValue(N03)).doubleValue(), MAXE);
		assertEquals(0.5, vset3.getUtility(new NumberValue(N07)).doubleValue(),
				MAXE);
		assertEquals(0, vset3.getUtility(new NumberValue(N09)).doubleValue(),
				MAXE);

	}

}
